package se.hig.jdalmas;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import se.sics.prologbeans.PBTerm;

public class Act extends GenericComponent {
	protected String actID;
	protected String [] actParams;

	/**
	 * @param actID The name of this Act.
	 */
	public Act(String actID) {
		this(actID,null);
	}

	/**
	 * @param actID The name of this Act.
	 * @param actParams The parameter list of this Act.
	 */
	public Act(String actID, String [] actParams) {
		this.actID = actID;
		this.actParams = actParams;
//		this.prologTerm = createActTerm(actID,actParam);
	}

	/**
	 * @return the actID
	 */
	public String getActID() {
		return actID;
//		return prologTerm.getName();
	}

	/**
	 * @return the actParam
	 */
	public String [] getActParams() {
		return actParams;
//		if (prologTerm.isCompound())
//			return prologTerm.getArgument(1).getName();
//		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		if (this.actParams == null)
			return this.actID;
		return this.actID + "(" + toString(this.actParams) + ")";
	}
	
	public static Act createAct(String actID, String [] actParams) {
		if (actParams == null)
			return new Act(actID);
		return new Act(actID,actParams);
	}

	public static PBTerm createActTerm(String actID, String [] actParams) {
		if (actParams == null)
			return PBTerm.makeAtom(actID);
		else {
			PBTerm [] paramsTerm = new PBTerm[actParams.length];
			for(int i=0; i<paramsTerm.length; i++) {
				Long intParam = parseLong(actParams[i]);
				if (intParam != null)
					paramsTerm[i] = PBTerm.makeTerm(intParam);
				else {
					Double doubleParam = parseDouble(actParams[i]);
					if (doubleParam != null)
						paramsTerm[i] = PBTerm.makeTerm(doubleParam);
					else
						paramsTerm[i] = PBTerm.makeAtom(actParams[i]);
				}				
			}
			return PBTerm.makeTerm(actID,paramsTerm);
		}
	}

	private static Long parseLong(String s) {
		try {
			long l = Long.parseLong(s);
			return new Long(l);
		} catch (NumberFormatException nfe) {
			return null;
		}
	}

	private static Double parseDouble(String s) {
		try {
			double d = Double.parseDouble(s);
			return new Double(d);
		} catch (NumberFormatException nfe) {
			return null;
		}
	}

	public static Act createAct(PBTerm actTerm) {
		String actID = actTerm.getName();
		if (actTerm.isAtom())
			return new Act(actID);
		String [] actParams = new String[actTerm.getArity()];
		for(int i=0; i<actParams.length; i++) {
			PBTerm paramTerm = actTerm.getArgument(i+1);
			actParams[i] = null;
			if (paramTerm.isInteger())
				actParams[i] = String.valueOf(paramTerm.intValue());
			else if (paramTerm.isFloat())
				actParams[i] = String.valueOf(paramTerm.floatValue());
			else
				actParams[i] = paramTerm.getName();			
		}
		return createAct(actID,actParams);
	}
	
	public static PBTerm createActListPrologTerm(List<Act> actList) {
		Stack<Act> actStack = new Stack<Act>();
		for(Act act : actList)
			actStack.push(act);
		PBTerm actsTerm = PBTerm.NIL;
		while (!actStack.isEmpty()) {
			Act act = actStack.pop();
			actsTerm = PBTerm.makeTerm(createActTerm(act.actID,act.actParams),actsTerm);
//			term = PBTerm.makeTerm(act.getPrologTerm(),term);
		}
		return actsTerm;
	}

	public static List<Act> createActList(PBTerm actListTerm) {
		List<Act> actList = new LinkedList<Act>();
		while (!actListTerm.isEmptyList()) {
			Act act = createAct(actListTerm.head());
//			Act act = new Act(actListTerm.head());
			actList.add(act);
			actListTerm = actListTerm.tail();
		}
		return actList;
	}
	
	public static List<Act> createEmptyActList() {
		return new LinkedList<Act>();
	}
	
}