package se.hig.jdalmas;

import java.util.List;
import java.util.Map;

import se.sics.prologbeans.PBTerm;

public class DDALMAS extends GenericComponent {
	protected java.util.List<Agent> agentList;
	protected java.util.List<Act> actList;
	protected java.util.Map<String,Predicate> predicates;

	/**
	 * @param agentList
	 * @param actionList
	 * @param predicates
	 */
	public DDALMAS(List<Agent> agentList, List<Act> actList, Map<String, Predicate> predicates) {
		this.agentList = agentList;
		this.actList = actList;
		this.predicates = predicates;
	}

	/**
	 * @return the actionSet
	 */
	public List<Act> getActList() {
		return actList;
	}

	/**
	 * @param actionList the actionSet to set
	 */
	public void setActList(List<Act> actList) {
		this.actList = actList;
	}

	/**
	 * @return the agentList
	 */
	public List<Agent> getAgentList() {
		return agentList;
	}

	/**
	 * @param agentList the agentList to set
	 */
	public void setAgentList(List<Agent> agentList) {
		this.agentList = agentList;
	}

	/**
	 * @return the predicates
	 */
	public java.util.Map<String, Predicate> getPredicates() {
		return predicates;
	}

	/**
	 * @param predicates the predicates to set
	 */
	public void setPredicates(java.util.Map<String, Predicate> predicates) {
		this.predicates = predicates;
	}

	public Agent findAgent(String agentID) {
		for (Agent agent : this.getAgentList()) {
			if (agentID.equals(agent.getAgentID()))
				return agent;
		}
		return null;
	}

	public Act findAct(String actID, String [] actParameters) {
		for (Act act : this.getActList()) {
			if (actID.equals(act.getActID()) && actParameters.equals(act.getActParams()))
				return act;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "(" + this.agentList + "," + this.actList + this.predicates + ")";
	}
	
	public static DDALMAS createDDALMAS(PBTerm dDALMASTerm) {
		List<Agent> agentList = Agent.createAgentList(dDALMASTerm.getArgument(1));
		List<Act> actionList = Act.createActList(dDALMASTerm.getArgument(2));
		Map<String,Predicate> predicates = Predicate.createEmptyPredicateMap();
//		predicates.put(ACT_RESULT_PREDICATE,Predicate.createPredicate(dDALMASTerm.getArgument(3)));
		predicates.put(FEASIBLE_ACT_PREDICATE,Predicate.createPredicate(dDALMASTerm.getArgument(4)));
		predicates.put(PERMISSIBLE_ACT_PREDICATE,Predicate.createPredicate(dDALMASTerm.getArgument(5)));
		predicates.put(UTILITY_FUNCTION_PREDICATE,Predicate.createPredicate(dDALMASTerm.getArgument(6)));
		predicates.put(CHOICE_SET_PREDICATE,Predicate.createPredicate(dDALMASTerm.getArgument(7)));
		predicates.put(TURN_OPERATOR_PREDICATE,Predicate.createPredicate(dDALMASTerm.getArgument(8)));
		predicates.put(TIE_BREAK_PREDICATE,Predicate.createPredicate(dDALMASTerm.getArgument(9)));
		return new DDALMAS(agentList,actionList,predicates);
	}

	public static DDALMAS createEmptyDDALMAS() {
		List<Agent> agentList = Agent.createEmptyAgentList();
		List<Act> actionList = Act.createEmptyActList();
		Map<String,Predicate> predicates = Predicate.createEmptyPredicateMap();
		return createDDALMAS(agentList,actionList,predicates);
	}

	public static DDALMAS createDDALMAS(List<Agent> agentList, 
			List<Act> actionList, Map<String,Predicate> predicates) {
		return new DDALMAS(agentList,actionList,predicates);
	}

	public static PBTerm createPrologTerm(DDALMAS dDALMAS) {
		Map<String, Predicate> predicates = dDALMAS.getPredicates();
		List<Agent> agentList = dDALMAS.getAgentList();
		List<Act> actionList = dDALMAS.getActList();
		PBTerm [] args = new PBTerm[9];
		args[0] = Agent.createAgentListPrologTerm(agentList);
		args[1] = Act.createActListPrologTerm(actionList);
		// TODO: Fix structure of DDALMAS!
		args[2] = PBTerm.NIL;
//		args[2] = Predicate.createPrologTerm(predicates.get(ACT_RESULT_PREDICATE));
		args[3] = Predicate.createPrologTerm(predicates.get(FEASIBLE_ACT_PREDICATE));
		args[4] = Predicate.createPrologTerm(predicates.get(PERMISSIBLE_ACT_PREDICATE));
		args[5] = Predicate.createPrologTerm(predicates.get(UTILITY_FUNCTION_PREDICATE));
		args[6] = Predicate.createPrologTerm(predicates.get(CHOICE_SET_PREDICATE));
		args[7] = Predicate.createPrologTerm(predicates.get(TURN_OPERATOR_PREDICATE));
		args[8] = Predicate.createPrologTerm(predicates.get(TIE_BREAK_PREDICATE));
		return PBTerm.makeTerm(D_DALMAS,args);
	}

}