package se.hig.jdalmas;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import se.hig.jdalmas.tuples.Tuple2;
import se.sics.prologbeans.Bindings;
import se.sics.prologbeans.IllegalCharacterSetException;
import se.sics.prologbeans.PBTerm;
import se.sics.prologbeans.QueryAnswer;


public class DnrDALMASRun extends GenericComponent {
	private DnrDALMASSituation startSit = null;
	private List<Tuple2<Agent,Act>> agentActSequence = null;
	private DnrDALMASSituation finalSit = null;

	/**
	 * @param startSit
	 * @param agentActSequence
	 * @param finalSit
	 */
	protected DnrDALMASRun(DnrDALMASSituation startSit,
			List<Tuple2<Agent,Act>> agentActSequence, DnrDALMASSituation finalSit) {
		this.startSit = startSit;
		this.agentActSequence = agentActSequence;
		this.finalSit = finalSit;
	}

	public void setStartSit(DnrDALMASSituation startSit) {
		this.startSit = startSit;
	}

	/**
	 * @return the startSit
	 */
	public DnrDALMASSituation getStartSit() {
		return startSit;
	}

	/**
	 * @return the agentActSequence
	 */
	public List<Tuple2<Agent,Act>> getAgentActSequence() {
		return agentActSequence;
	}

	public void setFinalSit(DnrDALMASSituation finalSit) {
		this.finalSit = finalSit;
	}

	/**
	 * @return the finalSit
	 */
	public DnrDALMASSituation getFinalSit() {
		return finalSit;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuffer sBuf = new StringBuffer();
		for(Tuple2<Agent,Act> pair : agentActSequence) {
			sBuf.append(pair.first + "-" + pair.second + "\n");
		}
		return sBuf.toString();
	}

	public static DnrDALMASRun kEventRun(int k, DnrDALMASSituation startSit) {
		final String InitialSit = "InitialSit";
		final String K = "K";
		final String Run = "Run";
		PBTerm runTerm = null;
		try {
			Bindings b = new Bindings();
			b.bind(InitialSit,DnrDALMASSituation.createPrologTerm(startSit));
			b.bind(K,k);
			String queryString = 
				K_EVENT_RUN + "(" + InitialSit + "," + K + "," + Run + ")";
			System.out.println("?- " + queryString + ".");
			connect();
			QueryAnswer answer = 
				prologSessionHandler.getSession().executeQuery(queryString,b);
			System.out.println("=> " + answer);
			if ((answer == null) || answer.queryFailed()) {
				disconnect();
				return null;
			}
			runTerm = answer.getValue(Run);
//			System.out.println("runTerm = " + runTerm);
		} catch(IOException x) {
			x.printStackTrace();
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
		}
		disconnect();
		return createKEventRun(runTerm);
	}
	
	public static DnrDALMASRun createKEventRun(PBTerm runTerm) {
		PBTerm initialSitTerm = runTerm.getArgument(3);
		DnrDALMASSituation initialSit = DnrDALMASSituation.createSituation(initialSitTerm);
		List<Tuple2<Agent,Act>> agentActSequence = new LinkedList<Tuple2<Agent,Act>>();
		PBTerm pairsTerm = runTerm.getArgument(2);
		while (!pairsTerm.isEmptyList()) {
			Tuple2<Agent,Act> pair = createAgentActPair(pairsTerm.head());
			agentActSequence.add(0,pair);
			pairsTerm = pairsTerm.tail();
		}
		PBTerm finalSitTerm = runTerm.getArgument(1);
		DnrDALMASSituation finalSit = DnrDALMASSituation.createSituation(finalSitTerm);
		return new DnrDALMASRun(initialSit,agentActSequence,finalSit);
	}

	public static Tuple2<Agent,Act> createAgentActPair(PBTerm agentActTerm) {
		PBTerm agentTerm = agentActTerm.getArgument(1);
		Agent agent = Agent.createAgent(agentTerm);
		PBTerm actTerm = agentActTerm.getArgument(2);
		Act act = Act.createAct(actTerm);
		return new Tuple2<Agent,Act>(agent,act);
	}

}