package se.hig.jdalmas;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import se.hig.jdalmas.norms.Norm;
import se.hig.jdalmas.norms.StateCondition;
import se.hig.jdalmas.tuples.Tuple2;
import se.hig.jdalmas.tuples.Tuple3;
import se.hig.jdalmas.tuples.Tuple4;
import se.hig.jdalmas.tuples.Tuple7;
import se.sics.prologbeans.Bindings;
import se.sics.prologbeans.IllegalCharacterSetException;
import se.sics.prologbeans.PBTerm;
import se.sics.prologbeans.QueryAnswer;

public class DnrDALMASSituation extends GenericComponent {
	protected Agent agentToMove;
	protected DnrDALMASState state;

	/**
	 * @param agentToMove
	 * @param state
	 */
	public DnrDALMASSituation(Agent agentToMove, DnrDALMASState state) {
		this.agentToMove = agentToMove;
		this.state = state;
	}

	/**
	 * @return the agentToMove
	 */
	public Agent getAgentToMove() {
		return agentToMove;
	}

	/**
	 * @param agentToMove the agentToMove to set
	 */
	public void setAgentToMove(Agent agentToMove) {
		this.agentToMove = agentToMove;
	}

	/**
	 * @return the state
	 */
	public DnrDALMASState getState() {
		return state;
	}

	/**
	 * @param state the state to set
	 */
	public void setState(DnrDALMASState state) {
		this.state = state;
	}

	public Agent findAgent(String agentID) {
		return this.getState().findAgent(agentID);
	}

	public Act findAct(String actID, String [] actParameters) {
		return this.getState().findAct(actID,actParameters);
	}

	
	public KnowledgeBase getKnowledgeBase() {
		DnrDALMASState state = this.getState();
		return state.getKnowledgeBase();
	}

	public List<Agent> getAgentSet() {
		return this.getState().getDnrDALMAS().getDDALMAS().getAgentList();
	}

	public List<Act> getActSet() {
		return this.getState().getDnrDALMAS().getDDALMAS().getActList();
	}

	public Map<String, Predicate> getPredicates() {
		return this.getState().getDnrDALMAS().getDDALMAS().getPredicates();
	}

	public List<Norm> getNormSet() {
		return this.getState().getDnrDALMAS().getNormSystem().getNormSet();
	}

	public Map<String, StateCondition> getGroundSet() {
		return this.getState().getDnrDALMAS().getNormSystem().getGroundSet();
	}

	public Map<String, StateCondition> getConsequenceSet() {
		return this.getState().getDnrDALMAS().getNormSystem().getConsequenceSet();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "<" + this.agentToMove + "," + this.state + ">";
	}

	public static PBTerm createPrologTerm(DnrDALMASSituation situation) {
		String agentToMove = situation.getAgentToMove().getAgentID();
		PBTerm [] args = {
				PBTerm.makeAtom(agentToMove),
				DnrDALMASState.createPrologTerm(situation.getState())
		};
		return PBTerm.makeTerm(DNR_DALMAS_SIT,args);
	}

	public static DnrDALMASSituation createSituation(PBTerm prologTerm) {
		PBTerm agentTerm = prologTerm.getArgument(1);
		PBTerm stateTerm = prologTerm.getArgument(2);
		DnrDALMASState state = DnrDALMASState.createState(stateTerm);
		Agent agentToMove = state.findAgent(agentTerm.toString());
		return createSituation(agentToMove,state);
	}

	private static List<Tuple2<Agent,Act>> createFeasibleActSet(PBTerm prologTerm) {
		return createAgentActPairs(prologTerm);
	}
	
	private static List<Tuple2<Agent,Act>> createDeonticStructure(PBTerm prologTerm) {
		return createAgentActPairs(prologTerm);
	}
	
	private static List<Tuple2<Agent,Act>> createChoiceSet(PBTerm prologTerm) {
		return createAgentActPairs(prologTerm);
	}

	private static List<Tuple2<Agent,Act>> createAgentActPairs(PBTerm agentActPairsTerm) {
		List<Tuple2<Agent,Act>> agentActPairs = new LinkedList<Tuple2<Agent,Act>>();
		while (!agentActPairsTerm.isEmptyList()) {
			PBTerm pairTerm = agentActPairsTerm.head();
			Agent agent = Agent.createAgent(pairTerm.getArgument(1));
			Act act = Act.createAct(pairTerm.getArgument(2));
			agentActPairs.add(new Tuple2<Agent,Act>(agent,act));
			agentActPairsTerm = agentActPairsTerm.tail();
		}
		return agentActPairs;
	}

	private static List<Tuple3<String,Agent,Act>> createPreferenceStructure(PBTerm preferenceStructureTerm) {
		List<Tuple3<String,Agent,Act>> preferenceStructure = new LinkedList<Tuple3<String,Agent,Act>>();
		while (!preferenceStructureTerm.isEmptyList()) {
			PBTerm preferenceTerm = preferenceStructureTerm.head();
			String value = preferenceTerm.getArgument(1).toString();
			PBTerm agentActPairTerm = preferenceTerm.getArgument(2);
			Agent agent = Agent.createAgent(agentActPairTerm.getArgument(1));
			Act act = Act.createAct(agentActPairTerm.getArgument(2));
			preferenceStructure.add(new Tuple3<String,Agent,Act>(value,agent,act));
			preferenceStructureTerm = preferenceStructureTerm.tail();
		}
		return preferenceStructure;
	}

	public static DnrDALMASSituation createEmptySituation(String normSystemID) {
		return createSituation(null,DnrDALMASState.createEmptyState(normSystemID));
	}

	public static DnrDALMASSituation 
	createSituation(Agent agentToMove, DnrDALMASState state) {
		return new DnrDALMASSituation(agentToMove,state);
	}
	
	public static Tuple7<DnrDALMASSituation,List<Tuple2<Agent,Act>>,List<Tuple2<Agent,Act>>,
	List<Tuple3<String,Agent,Act>>,List<Tuple2<Agent,Act>>,Agent,Act> 
	nextSituation(DnrDALMASSituation currentSituation) {
		final String SIT = "Sit";
		final String NEXT_SIT = "NextSit";
		final String FAs = "FeasibleActSet";
		final String DS = "DeonticStructure";
		final String PS = "PreferenceStructure";
		final String CS = "ChoiceSet";
		final String W = "W";
		final String A = "A";
		PBTerm nextSituationTerm = null;
		PBTerm feasibleActsTerm = null;
		PBTerm deonticStructureTerm = null;
		PBTerm preferenceStructureTerm = null;
		PBTerm choiceSetTerm = null;
		PBTerm actingAgentTerm = null;
		PBTerm chosenActTerm = null;
		try {
			Bindings b = new Bindings();
			PBTerm situationTerm = createPrologTerm(currentSituation);
//			System.out.println(situationTerm);
			b.bind(SIT,situationTerm);
//			System.out.println(b);
			String queryString = NEXT_SITUATION + "(" + SIT + "," + NEXT_SIT + "," + 
				FAs + "," + DS + "," + PS + "," + CS + "," + W + "," + A + ")";
//			System.out.println("?- " + queryString + ".");
			connect();
			QueryAnswer answer = 
				prologSessionHandler.getSession().executeQuery(queryString,b);
//			System.out.println("=> " + answer);
			if ((answer == null) || answer.queryFailed()) {
				disconnect();
				return null;
			}
			nextSituationTerm = answer.getValue(NEXT_SIT);
//			System.out.println("nextSituationTerm = " + nextSituationTerm);
			feasibleActsTerm = answer.getValue(FAs);
//			System.out.println("feasibleActsTerm = " + feasibleActsTerm);
			deonticStructureTerm = answer.getValue(DS);
//			System.out.println("deonticStructureTerm = " + deonticStructureTerm);
			preferenceStructureTerm = answer.getValue(PS);
//			System.out.println("preferenceStructureTerm = " + preferenceStructureTerm);
			choiceSetTerm = answer.getValue(CS);
//			System.out.println("choiceSetTerm = " + choiceSetTerm);
			actingAgentTerm = answer.getValue(W);
//			System.out.println("actingAgentTerm = " + actingAgentTerm);
			chosenActTerm = answer.getValue(A);
//			System.out.println("chosenActTerm = " + chosenActTerm);
		} catch(IOException x) {
			x.printStackTrace();
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
		}
		disconnect();
		DnrDALMASSituation nextSituation = createSituation(nextSituationTerm);
		// TODO: Fix feasibleActs, deonticStructure, preferenceStructure and choiceSet
		List<Tuple2<Agent,Act>> feasibleActs = null;
		List<Tuple2<Agent,Act>> deonticStructure = null;
		List<Tuple3<String,Agent,Act>> preferenceStructure = null;
		List<Tuple2<Agent,Act>> choiceSet = createChoiceSet(choiceSetTerm);
		Agent actingAgent = Agent.createAgent(actingAgentTerm);
		Act chosenAct = Act.createAct(chosenActTerm);
		return new Tuple7<DnrDALMASSituation,List<Tuple2<Agent,Act>>,List<Tuple2<Agent,Act>>,
		List<Tuple3<String,Agent,Act>>,List<Tuple2<Agent,Act>>,Agent,Act>(nextSituation,
				feasibleActs,deonticStructure,preferenceStructure,choiceSet,actingAgent,chosenAct);
	}
	
	public static List<Tuple2<Agent,Act>> feasibleActSet(DnrDALMASSituation currentSituation) {
		final String SIT = "Sit";
		final String FAS = "FeasibleActs";
		PBTerm feasibleActsTerm = null;
		try {
			Bindings b = new Bindings();
			b.bind(SIT,createPrologTerm(currentSituation));
//			System.out.println(b);
			String queryString = FEASIBLE_ACTS + "(" + SIT + "," + FAS + ")";
//			System.out.println("?- " + queryString + ".");
			connect();
			QueryAnswer answer = 
				prologSessionHandler.getSession().executeQuery(queryString,b);
//			System.out.println("=> " + answer);
			if ((answer == null) || answer.queryFailed()) {
				disconnect();
				return null;
			}
			feasibleActsTerm = answer.getValue(FAS);
//			System.out.println("feasibleActsTerm = " + feasibleActsTerm);
		} catch(IOException x) {
			x.printStackTrace();
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
		}
		disconnect();
		List<Tuple2<Agent,Act>> feasibleActs = createFeasibleActSet(feasibleActsTerm);
		return feasibleActs;
	}
	
	public static Tuple4<List<Tuple2<Agent,Act>>,List<Tuple2<Agent,Act>>,
	List<Tuple3<String,Agent,Act>>,List<Tuple2<Agent,Act>>> choiceSet(DnrDALMASSituation currentSituation) {
		final String SIT = "Sit";
		final String FAS = "FeasibleActs";
		final String DS = "DeonticStructure";
		final String PS = "PreferenceStructure";
		final String CS = "ChoiceSet";
		PBTerm feasibleActsTerm = null;
		PBTerm deonticStructureTerm = null;
		PBTerm preferenceStructureTerm = null;
		PBTerm choiceSetTerm = null;
		try {
			Bindings b = new Bindings();
			b.bind(SIT,createPrologTerm(currentSituation));
//			System.out.println(b);
			String queryString = CHOICE_SET + "(" + SIT + "," +
				FAS + "," + DS + "," + PS + "," + CS + ")";
			System.out.println("?- " + queryString + ".");
			connect();
			QueryAnswer answer = 
				prologSessionHandler.getSession().executeQuery(queryString,b);
//			System.out.println("=> " + answer);
			if ((answer == null) || answer.queryFailed()) {
				disconnect();
				return null;
			}
			feasibleActsTerm = answer.getValue(FAS);
//			System.out.println("feasibleActsTerm = " + feasibleActsTerm);
			deonticStructureTerm = answer.getValue(DS);
//			System.out.println("deonticStructureTerm = " + deonticStructureTerm);
			preferenceStructureTerm = answer.getValue(PS);
//			System.out.println("preferenceStructureTerm = " + preferenceStructureTerm);
			choiceSetTerm = answer.getValue(CS);
//			System.out.println("choiceSetTerm = " + choiceSetTerm);
		} catch(IOException x) {
			x.printStackTrace();
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
		}
		disconnect();
		List<Tuple2<Agent,Act>> feasibleActs = createFeasibleActSet(feasibleActsTerm); 
		List<Tuple2<Agent,Act>> deonticStructure = createDeonticStructure(deonticStructureTerm);
		List<Tuple3<String,Agent,Act>> preferenceStructure = createPreferenceStructure(preferenceStructureTerm);
		List<Tuple2<Agent,Act>> choiceSet = createChoiceSet(choiceSetTerm);
		return new Tuple4<List<Tuple2<Agent,Act>>,List<Tuple2<Agent,Act>>,List<Tuple3<String,Agent,Act>>,
		List<Tuple2<Agent,Act>>>(feasibleActs,deonticStructure,preferenceStructure,choiceSet);
	}

}