package se.hig.jdalmas;

import java.util.Map;
import java.util.TreeMap;

import se.sics.prologbeans.PBTerm;

public class Predicate extends GenericComponent {
	protected String module;
	protected String functor;
	protected int arity;

	/**
	 * @param module
	 * @param functor
	 * @param arity
	 */
	public Predicate(String module, String functor, int arity) {
		this.module = module;
		this.functor = functor;
		this.arity = arity;
	}

	/**
	 * @param functor
	 * @param arity
	 */
	public Predicate(String functor, int arity) {
		this("user", functor, arity);
	}

	/**
	 * @return the module
	 */
	public String getModule() {
		return module;
	}

	/**
	 * @return the arity
	 */
	public int getArity() {
		return arity;
	}

	/**
	 * @return the functor
	 */
	public String getFunctor() {
		return functor;
	}

	/* (non-Javadoc)
	 * @see se.hig.jdalmas.GenericDnrDALMASTerm#toString()
	 */
	@Override
	public String toString() {
		return this.module + ":" + this.functor + "/" + this.arity;
	}

	public static Map<String,Predicate> createEmptyPredicateMap() {
		return new TreeMap<String,Predicate>();
	}

	public static Predicate createPredicate(String functor, int arity) {
		return new Predicate(functor,arity);
	}

	public static Predicate createPredicate(String module, String functor, int arity) {
		return new Predicate(module,functor,arity);
	}

	public static PBTerm createPrologTerm(Predicate predicate) {
		PBTerm [] args2 = new PBTerm[2];
		args2[0] = PBTerm.makeAtom(predicate.getFunctor());
		args2[1] = PBTerm.makeTerm(predicate.getArity());
		PBTerm term2 = PBTerm.makeTerm("/", args2);
		PBTerm [] args1 = new PBTerm[2];
		args1[0] = PBTerm.makeAtom(predicate.getModule());
		args1[1] = term2;
		return PBTerm.makeTerm(":", args1);
	}

	public static Predicate createPredicate(PBTerm predicateTerm) {
		PBTerm moduleTerm = predicateTerm.getArgument(1);
		String module = moduleTerm.toString();
		PBTerm functorAndArityTerm = predicateTerm.getArgument(2);
		PBTerm functorTerm = functorAndArityTerm.getArgument(1);
		String functor = functorTerm.toString();
		PBTerm arityTerm = functorAndArityTerm.getArgument(2);
		int arity = (int)arityTerm.intValue();
		return new Predicate(module, functor, arity);
	}

}