package se.hig.jdalmas.control;

import java.util.List;
import java.util.Map;

import se.hig.jdalmas.Act;
import se.hig.jdalmas.Agent;
import se.hig.jdalmas.DnrDALMASRun;
import se.hig.jdalmas.DnrDALMASSituation;
//import se.hig.jdalmas.Knowledge;
import se.hig.jdalmas.KnowledgeBase;
import se.hig.jdalmas.Predicate;
import se.hig.jdalmas.exceptions.DnrDALMASException;
import se.hig.jdalmas.norms.Consequence;
import se.hig.jdalmas.norms.Ground;
import se.hig.jdalmas.norms.ElementaryConsequence;
import se.hig.jdalmas.norms.ElementaryGround;
import se.hig.jdalmas.norms.Norm;
import se.hig.jdalmas.norms.NormSystem;
import se.hig.jdalmas.norms.Operator;
import se.hig.jdalmas.norms.MoveOperator;
import se.hig.jdalmas.norms.TypeOperator;
import se.hig.jdalmas.norms.StateCondition;
import se.hig.jdalmas.tuples.Tuple2;
import se.hig.jdalmas.tuples.Tuple3;
import se.hig.jdalmas.tuples.Tuple4;
import java.util.Observable;

public class DnrDALMASController extends Observable {
	protected DnrDALMASRun run = null;
	protected DnrDALMASSituation currentSituation = null;
	protected List<Tuple2<Agent,Act>> feasibleActSet = null;
	protected List<Tuple2<Agent,Act>> deonticStructure = null;
	protected List<Tuple3<String,Agent,Act>> preferenceStructure = null;
	protected List<Tuple2<Agent,Act>> choiceSet = null;
	protected Agent actingAgent = null;
	protected String normSystemID = null;

	protected Operator [] groundOps = MoveOperator.M;
	protected Operator [] consequenceOps = TypeOperator.T;

	public DnrDALMASController(String host, int port, String normSystemID) {
		if (port<0)
			throw new DnrDALMASException("Error: illegal port value (" + port + ")");
		PrologSessionHandler.getInstance().createSession(host,port);
		this.normSystemID = normSystemID;
		currentSituation = 
			DnrDALMASSituation.createEmptySituation(normSystemID);
		addPredicate(Predicate.FEASIBLE_ACT_PREDICATE,
				"dnrDALMAS","feasibleAct_simple",3);
		addPredicate(Predicate.PERMISSIBLE_ACT_PREDICATE, 
				"dnrDALMAS","permissibleAct_simple",3);
		addPredicate(Predicate.UTILITY_FUNCTION_PREDICATE,"user","?",4);
		addPredicate(Predicate.CHOICE_SET_PREDICATE, 
				"dnrDALMAS","choiceSet_simple",5);
		addPredicate(Predicate.TURN_OPERATOR_PREDICATE, 
				"dnrDALMAS","turnOperator_simple",2);
		addPredicate(Predicate.TIE_BREAK_PREDICATE, 
				"dnrDALMAS","tieBreaker_simple",4);
	}
	
	public void addAgent(String agentID) {
		Agent agent = Agent.createAgent(agentID);
		currentSituation.getState().getDnrDALMAS().getDDALMAS().getAgentList().add(agent);
		setChanged();
		notifyObservers(agent);
	}

	public void addAct(String actID, String [] actParams) {
		Act act = Act.createAct(actID,actParams);
		currentSituation.getState().getDnrDALMAS().getDDALMAS().getActList().add(act);
		setChanged();
		notifyObservers(act);
	}

	public void addPredicate(String key, String functor, int arity) {
		Predicate predicate = Predicate.createPredicate(functor,arity);
		currentSituation.getState().getDnrDALMAS().getDDALMAS().getPredicates().put(key,predicate);
		setChanged();
		notifyObservers(predicate);
	}

	public void addPredicate(String key, String module, String functor, int arity) {
		Predicate predicate = 
			Predicate.createPredicate(module,functor,arity);
		currentSituation.getState().getDnrDALMAS().getDDALMAS().getPredicates().put(key,predicate);
		setChanged();
		notifyObservers(predicate);
	}

//	public void addConsequenceToConsequenceSet(String consequenceID, String module, 
//			String functor, int parameters, int agentArguments) {
	public void addConsequenceToConsequenceSet(String consequenceID, String module, 
			String functor, int parameters, int agentArguments) {
		StateCondition stateCond = 
			StateCondition.createStateCondition(module,functor,parameters,agentArguments);
		NormSystem normSystem = currentSituation.getState().getDnrDALMAS().getNormSystem();
		normSystem.addConsequenceToConsequenceSet(consequenceID,stateCond);
		currentSituation.getState().getDnrDALMAS().setNormSystem(normSystem);
		setChanged();
		notifyObservers(normSystem);
	}

	public void addElementaryNorm(String normID,
			int groundOpIndex, String groundCondition, String [] groundConditionParameters,
			int consequenceOpIndex, String consequenceCondition, String [] consequenceConditionParameters) {
		if (normID == null || normID.equals(""))
			throw new DnrDALMASException("Norm ID must not be empty");
		//		if (cfDALMAS.containsNorm(normID))
		//			throw new DnrDALMASException("Norm " + normID + " already exists");
		Operator groundOp = groundOps[groundOpIndex];
		ElementaryGround ground = 
			Ground.createElementaryGround(currentSituation,
					groundOp,groundCondition,groundConditionParameters);
//		Operator consequenceOp = consequenceOps[consequenceOpIndex-1];
		Operator consequenceOp = Operator.createOperator(Operator.TYPE,consequenceOpIndex + "");
		ElementaryConsequence consequence = 
			Consequence.createElementaryConsequence(this.currentSituation,
					consequenceOp,consequenceCondition,consequenceConditionParameters);
		Norm norm = 
			NormSystem.createElementaryNorm(normSystemID,normID,ground,consequence);
		NormSystem normSystem = currentSituation.getState().getDnrDALMAS().getNormSystem();
		normSystem.addNorm(norm);
		currentSituation.getState().getDnrDALMAS().setNormSystem(normSystem);
		setChanged();
		notifyObservers(normSystem);
	}

//	public void addGroundToGroundSet(String groundID, String module, String functor, 
//	int parameters, int agentArguments) {
	public void addGroundToGroundSet(String groundID, String module, String functor, 
			int parameters, int agentArguments) {
		if (groundID == null || groundID.equals(""))
			throw new DnrDALMASException("Ground ID must not be empty");
		//		if (cfDALMAS.containsGround(groundID))
		//			throw new DnrDALMASException("Ground " + groundID + " already exists");
		StateCondition stateCond = 
			StateCondition.createStateCondition(module,functor,parameters,agentArguments);
		NormSystem normSystem = currentSituation.getState().getDnrDALMAS().getNormSystem();
		normSystem.addGroundToGroundSet(groundID,stateCond);
		currentSituation.getState().getDnrDALMAS().setNormSystem(normSystem);
		setChanged();
		notifyObservers(normSystem);
	}

	public void createEmptySituation() {
		this.currentSituation = 
			DnrDALMASSituation.createEmptySituation(normSystemID);
	}

	public void setAgentToMove(String agentID) {
		Agent agentToMove = 
			currentSituation.findAgent(agentID);
		if (agentToMove == null)
			throw new DnrDALMASException("Error: no such agent");
		currentSituation.setAgentToMove(agentToMove);
		setChanged();
		notifyObservers(currentSituation);
	}

	public String getAgentToMove() {
		if (currentSituation.getAgentToMove() != null)
			return currentSituation.getAgentToMove().getAgentID();
		return null;
	}

	public boolean containsAgent(String agentID) {
		if (currentSituation.findAgent(agentID) == null)
			return false;
		return true;
	}

	public boolean containsAct(String actID, String [] actParameters) {
		if (currentSituation.findAct(actID,actParameters) == null)
			return false;
		return true;
	}
	
	public String [] getAgentSet() {
		List<Agent> agentSet = currentSituation.getAgentSet();
		String [] agents = new String[agentSet.size()];
		int i=0;
		for (Agent agent : agentSet) {
			agents[i] = agent.toString();
			i++;
		}
		return agents;
	}

	public String [] getActSet() {
		List<Act> actSet = currentSituation.getActSet();
		String [] acts = new String[actSet.size()];
		int i=0;
		for (Act act : actSet) {
			acts[i] = act.toString();
			i++;
		}
		return acts;
	}

	public String getPredicateModule(String predicate) {
		Map<String,Predicate> predicateMap = currentSituation.getPredicates();
		return predicateMap.get(predicate).getModule();
	}

	public String getPredicateFunctor(String predicate) {
		Map<String,Predicate> predicateMap = currentSituation.getPredicates();
		return predicateMap.get(predicate).getFunctor();
	}
	
	public int getPredicateArity(String predicate) {
		Map<String,Predicate> predicateMap = currentSituation.getPredicates();
		return predicateMap.get(predicate).getArity();
	}

	public String [] getNormSet() {
		List<Norm> normSet = currentSituation.getNormSet();
		String [] norms = new String[normSet.size()];
		int i=0;
		for (Norm norm : normSet) {
			norms[i] = 
				norm.getNormID() + ": <" + norm.getGround().toString() + "," + norm.getConsequence().toString() + ">";
			i++;
		}
		return norms;
	}

	public Object [] getGroundOps() {
		return groundOps;
	}

	public Object [] getGroundSet() {
		Map<String,StateCondition> groundSet = currentSituation.getGroundSet();
		return groundSet.entrySet().toArray();
	}

	public Object [] getGroundIDSet() {
		Map<String,StateCondition> groundSet = currentSituation.getGroundSet();
		return groundSet.keySet().toArray();
	}

	public Object [] getConsequenceOps() {
		return consequenceOps;
	}

	public Object [] getConsequenceSet() {
		Map<String,StateCondition> consequenceSet = currentSituation.getConsequenceSet();
		return consequenceSet.entrySet().toArray();
	}

	public Object [] getConsequenceIDSet() {
		Map<String,StateCondition> consequenceSet = currentSituation.getConsequenceSet();
		return consequenceSet.keySet().toArray();
	}

	public void replaceCurrentKnowledgeBase(KnowledgeBase knowledgeBase) {
		currentSituation.getState().setKnowledgeBase(knowledgeBase);
		setChanged();
		notifyObservers(knowledgeBase);
	}
	
	public KnowledgeBase getCurrentKnowledgeBase() {
		return currentSituation.getState().getKnowledgeBase();
	}

	public DnrDALMASSituation getCurrentSituation() {
		return currentSituation;
	}

	public List<Tuple2<Agent,Act>> getFeasibleActSet() {
		return feasibleActSet;
	}

	public List<Tuple2<Agent,Act>> getDeonticStructure() {
		return deonticStructure;
	}

	public List<Tuple3<String,Agent,Act>> getPreferenceStructure() {
		return preferenceStructure;
	}

	public List<Tuple2<Agent,Act>> getChoiceSet() {
		return choiceSet;
	}

	public DnrDALMASRun getLatestRun() {
		return run;
	}

	public void nextSituation() throws DnrDALMASException {
		if (currentSituation.getAgentToMove() == null)
			throw new DnrDALMASException("Agent to move not initialized");
		currentSituation = DnrDALMASSituation.nextSituation(currentSituation).first;
		setChanged();
		notifyObservers(currentSituation);
	}
	
	public void feasibleActSet() throws DnrDALMASException {
		if (currentSituation.getAgentToMove() == null)
			throw new DnrDALMASException("Agent to move not initialized");
		feasibleActSet = DnrDALMASSituation.feasibleActSet(currentSituation);
	}

	public void choiceSet() throws DnrDALMASException {
		if (currentSituation.getAgentToMove() == null)
			throw new DnrDALMASException("Agent to move not initialized");
		Tuple4<List<Tuple2<Agent,Act>>,List<Tuple2<Agent,Act>>,List<Tuple3<String,Agent,Act>>,
		List<Tuple2<Agent,Act>>> result = DnrDALMASSituation.choiceSet(currentSituation);
		feasibleActSet = result.first;
		deonticStructure = result.second;
		preferenceStructure = result.third;
		choiceSet = result.fourth;
	}

	public void kEventRun(int k) {
		if (currentSituation.getAgentToMove() == null)
			throw new DnrDALMASException("Agent to move not initialized");
		run = DnrDALMASRun.kEventRun(k,currentSituation);
		if(run==null)
			currentSituation = null;
		else
			currentSituation = run.getFinalSit();
		setChanged();
		notifyObservers(currentSituation);
	}
}
