package se.hig.jdalmas.control;

import java.io.IOException;

import se.sics.prologbeans.*;

public class PrologSessionHandler {
	private PrologSession session = null;
	private String queryString = "shutdown(no_connections)";
	
	private static PrologSessionHandler singletonSessionHandler = 
		new PrologSessionHandler();

	/**
	 * Dold konstruktor.
	 */
	private PrologSessionHandler() {}

	/**
	 * Returnerar en referens till PrologSessionHandler-objektet.
	 */
	public static PrologSessionHandler getInstance() {
		return singletonSessionHandler;
	}
	
	/**
	 * @return the Prolog session
	 */
	public PrologSession getSession() {
		return session;
	}

	public void createSession(String host, int port) {
		System.out.print("* Creating Prolog session [" + host + ":" + port + "]... ");
		this.session = new PrologSession();
		this.session.setHost(host);
		this.session.setPort(port);
		// TODO: add timeout attribute
//		this.session.setTimeout(0);
		System.out.println("Done!");		
	}

	public boolean shutdown() {
		boolean shutdown = true;
		QueryAnswer queryAnswer = null;
		try {
//			System.out.print("* Connecting... ");
			this.session.connect();
//			System.out.println("Done!");
			System.out.println("?- " + queryString + ".");
			queryAnswer = this.session.executeQuery(queryString);
			System.out.println("=> " + queryAnswer);
			if ((queryAnswer == null) || queryAnswer.queryFailed())
				shutdown = false;
		} catch(IOException x) {
			x.printStackTrace();
			shutdown = false;
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
			shutdown = false;
		}
//		System.out.print("* Disconnecting... ");
		this.session.disconnect();
//		System.out.println("Done!");
		return shutdown;
	}

}