package se.hig.jdalmas.norms;

import java.util.Map;

import se.hig.jdalmas.DnrDALMAS;
import se.hig.jdalmas.DnrDALMASSituation;
import se.hig.jdalmas.DnrDALMASState;
import se.hig.jdalmas.GenericComponent;
import se.sics.prologbeans.PBTerm;

public abstract class Consequence extends GenericComponent {
	protected String booleanConnective = null;
	protected int arity = 0;
	protected SitCondition [] sitConds = null;
	
	public String getBooleanConnective() {
		return this.booleanConnective;
	}

	public SitCondition getSitCond(int i) {
		return this.sitConds[i];
	}

	public boolean isElementary() {
		return (this.booleanConnective == null);
	}
	
	public static PBTerm createPrologTerm(Consequence consequence) {
		if (consequence.isElementary())
			return SitCondition.createPrologTerm(consequence.getSitCond(0));
		// TODO: handle non-elementary consequences!
		return null;
	}

	public static ElementaryConsequence createConsequence(PBTerm consequenceTerm) {
		return new ElementaryConsequence(SitCondition.createSitCond(consequenceTerm));
	}

	public static ElementaryConsequence createElementaryConsequence(DnrDALMASSituation currentSituation, 
			Operator consequenceOp, String consequenceConditionID, String [] consequenceConditionParameters) {
		DnrDALMASState state = currentSituation.getState();
		DnrDALMAS dnrDALMAS = state.getDnrDALMAS();
		NormSystem normSystem = dnrDALMAS.getNormSystem();
		Map<String,StateCondition> consequenceSet = normSystem.getConsequenceSet();
		return createElementaryConsequence(consequenceOp,
				consequenceConditionID,consequenceConditionParameters,consequenceSet);
	}

	public static ElementaryConsequence createElementaryConsequence(Operator consequenceOp, 
			String consequenceConditionID, 
			String [] consequenceConditionParameters, 
			Map<String, StateCondition> consequenceSet) {
		if (!consequenceSet.containsKey(consequenceConditionID))
			return null;
		SitCondition sitCond = new SitCondition(consequenceOp,
				consequenceConditionID,consequenceConditionParameters);
		return new ElementaryConsequence(sitCond);
	}
}