package se.hig.jdalmas.norms;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

import se.hig.jdalmas.GenericComponent;
import se.sics.prologbeans.IllegalCharacterSetException;
import se.sics.prologbeans.PBTerm;
import se.sics.prologbeans.QueryAnswer;

public class NormSystem extends GenericComponent {
	protected String normSystemID;
	protected List<Norm> normSet;
	protected Map<String,StateCondition> groundSet;
	protected Map<String,StateCondition> consequenceSet;

	/**
	 * @param normSystemID
	 * @param normSet
	 * @param groundSet
	 * @param consequenceSet
	 */
	public NormSystem(String normSystemID, List<Norm> normSet, Map<String,StateCondition> groundSet, 
			Map<String,StateCondition> consequenceSet) {
		this.normSystemID = normSystemID;
		this.normSet = normSet;
		this.groundSet = groundSet;
		this.consequenceSet = consequenceSet;
	}

	/**
	 * @return the consequenceSet
	 */
	public Map<String,StateCondition> getConsequenceSet() {
		return consequenceSet;
	}

	/**
	 * @param consequenceSet the consequenceSet to set
	 */
	public void setConsequenceSet(Map<String,StateCondition> consequenceSet) {
		this.consequenceSet = consequenceSet;
	}

	/**
	 * @return the groundSet
	 */
	public Map<String,StateCondition> getGroundSet() {
		return groundSet;
	}

	/**
	 * @param groundSet the groundSet to set
	 */
	public void setGroundSet(Map<String,StateCondition> groundSet) {
		this.groundSet = groundSet;
	}

	/**
	 * @return the normSet
	 */
	public List<Norm> getNormSet() {
		return normSet;
	}

	/**
	 * @param normSet the normSet to set
	 */
	public void setNormSet(List<Norm> normSet) {
		this.normSet = normSet;
	}

	/**
	 * @param norm the norm to add
	 */
	public void addNorm(Norm norm) {
		this.normSet.add(norm);
	}

	public void addGroundToGroundSet(String groundID, StateCondition stateCond) {
		this.groundSet.put(groundID,stateCond);
	}

	public void addConsequenceToConsequenceSet(String consequenceID,
			StateCondition stateCond) {
		this.consequenceSet.put(consequenceID,stateCond);
	}

	/**
	 * @return the normSystemID
	 */
	public String getNormSystemID() {
		return normSystemID;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "<" 
		+ this.groundSet + ","
		+ this.consequenceSet + ","
		+ this.normSet
		+ ">";
	}

	public static PBTerm createEmptyGCTerm() {
		final String GCSet = "GCS";
		QueryAnswer queryAnswer = null;
		PBTerm gcSetTerm = null;
		try {
			String queryString = EMPTY_GCSET + "(" + GCSet + ")";
//			System.out.println("?- " + queryString + ".");
			connect();
			queryAnswer = prologSessionHandler.getSession().executeQuery(queryString);
//			System.out.println("=> " + queryAnswer);
			if ((queryAnswer != null) && !queryAnswer.queryFailed()) {
				gcSetTerm = queryAnswer.getValue(GCSet);
//				System.out.println("=> " + gcSetTerm);
			}
		} catch(IOException ioe) {
			ioe.printStackTrace();
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
		}
		disconnect();
		return gcSetTerm;
	}

	public static PBTerm createGCSetTerm(Map<String,StateCondition> gcSet) {
		PBTerm gcSetTerm = createEmptyGCTerm();
		for (Map.Entry<String,StateCondition> gcItem : gcSet.entrySet()) {
			gcSetTerm = addToGCSetTerm(gcItem,gcSetTerm);
		}
		return gcSetTerm;
	}

	public static PBTerm addToGCSetTerm(Entry<String, StateCondition> gcItem, 
			PBTerm gcSetTerm) {
		final String NewGCSet = "NewGCSet";
		QueryAnswer queryAnswer = null;
		try {
			String queryString = ADD_TO_GCSET + "(" + 
			gcItem.getKey() + "," + 
			gcSetTerm + "," + 
			gcItem.getValue().getModule() + ":" + gcItem.getValue().getFunctor() + "," +
			gcItem.getValue().getParameters() + "," +
			gcItem.getValue().getAgentArguments() + "," +
			NewGCSet + ")";
//			System.out.println("?- " + queryString + ".");
			connect();
			queryAnswer = prologSessionHandler.getSession().executeQuery(queryString);
//			System.out.println("=> " + queryAnswer);
			if ((queryAnswer != null) && !queryAnswer.queryFailed()) {
				gcSetTerm = queryAnswer.getValue(NewGCSet);
//				System.out.println("=> " + gcSetTerm);
			}
		} catch(IOException ioe) {
			ioe.printStackTrace();
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
		}
		disconnect();
		return gcSetTerm;
	}

	public static Map<String, StateCondition> createGCSet(PBTerm gcSetTerm) {
		final String ID = "ID";
		final String GC = "GC";
		final String PA = "PA";
		final String AA = "AA";
		final String RemainingGCSet = "RemainingGCSet";
		Map<String,StateCondition> gcSet = createEmptyGCSet();
		QueryAnswer queryAnswer = null;
		boolean queryFailed = false;
		while (!isEmptyGCTerm(gcSetTerm)) {
			try {
				String queryString = REMOVE_FIRST_FROM_GCSET + "("
				+ ID + "," 
				+ gcSetTerm.toString() + "," 
				+ GC + ","
				+ PA + "," 
				+ AA + ","
				+ RemainingGCSet + ")";
//				System.out.println("?- " + queryString + ".");
				connect();
				queryAnswer = prologSessionHandler.getSession().executeQuery(queryString);
//				System.out.println("=> " + queryAnswer);
				if ((queryAnswer == null) || queryAnswer.queryFailed()) {
					queryFailed = true;
					break;
				}
				String condID = queryAnswer.getValue(ID).toString();
				StateCondition stateCond = 
					StateCondition.createStateCondition(queryAnswer.getValue(GC), 
						queryAnswer.getValue(PA),queryAnswer.getValue(AA));
//				System.out.println("StateCondition: " + stateCond);
				gcSet.put(condID,stateCond);
				PBTerm remainingGCSetTerm = queryAnswer.getValue(RemainingGCSet);
//				System.out.println("RemainingGCSet: " + remainingGCSetTerm);
				// Move forward!
				gcSetTerm = remainingGCSetTerm;
			} catch(IOException ioe) {
				ioe.printStackTrace();
			} catch(IllegalCharacterSetException x) {
				x.printStackTrace();
			}
		}
		disconnect();
		if (queryFailed)
			return null;
		return gcSet;
	}

	public static Map<String,StateCondition> createEmptyGCSet() {
		return new TreeMap<String,StateCondition>();
	}

	public static boolean isEmptyGCTerm(PBTerm gcSetTerm) {
		if (gcSetTerm.isAtom() && gcSetTerm.toString().equals(EMPTY))
			return true;
		return false;
	}

	public static List<Norm> createEmptyNormSet() {
		return new LinkedList<Norm>();
	}

	public static List<Norm> createNormSet(PBTerm normSetTerm) {
		List<Norm> normSet = createEmptyNormSet();
		while (!normSetTerm.isEmptyList()) {
			Norm norm = createNorm(normSetTerm.head());
			normSet.add(0,norm); // Add norm first
			normSetTerm = normSetTerm.tail(); // Step forward
		}
		return normSet;
	}

	public static PBTerm createPrologTerm(List<Norm> normSet) {
		PBTerm term = PBTerm.NIL;
		for (Norm norm : normSet) {
			term = PBTerm.makeTerm(createPrologTerm(norm),term);
		}
		return term;
	}

	public static NormSystem createNormSystem(String id, List<Norm> normSet, 
			Map<String,StateCondition> groundSet, Map<String,StateCondition> consequenceSet) {
		return new NormSystem(id,normSet,groundSet,consequenceSet);
	}

	public static NormSystem createEmptyNormSystem(String normSystemID) {
		return createNormSystem(normSystemID,createEmptyNormSet(),
				NormSystem.createEmptyGCSet(),NormSystem.createEmptyGCSet());
	}

	public static PBTerm createPrologTerm(NormSystem normSystem) {
		PBTerm [] args = {
				NormSystem.createGCSetTerm(normSystem.getGroundSet()),
				NormSystem.createGCSetTerm(normSystem.getConsequenceSet()),
				NormSystem.createPrologTerm(normSystem.getNormSet())};
		return PBTerm.makeTerm(NORM_SYSTEM,args);
	}

	public static NormSystem createNormSystem(PBTerm normSystemTerm) {
		Map<String,StateCondition> groundSet = 
			NormSystem.createGCSet(normSystemTerm.getArgument(1));
		Map<String,StateCondition> consequenceSet = 
			NormSystem.createGCSet(normSystemTerm.getArgument(2));
		List<Norm> normSet = createNormSet(normSystemTerm.getArgument(3));
		return new NormSystem("_",normSet,groundSet,consequenceSet);
	}

	public static Norm createElementaryNorm(String normSystemID, String normID, 
			ElementaryGround ground, ElementaryConsequence consequence) {
		return new ElementaryNorm(normSystemID,normID,ground,consequence);
	}

	public static PBTerm createPrologTerm(Norm norm) {
		PBTerm [] args2 = { PBTerm.makeAtom(norm.getNormSystemID()), 
				PBTerm.makeAtom(norm.getNormID()) };
		PBTerm term = PBTerm.makeTerm("/",args2);
		PBTerm [] args1 = { term,Ground.createPrologTerm(norm.getGround()), 
				Consequence.createPrologTerm(norm.getConsequence())};
		return PBTerm.makeTerm(NORM,args1);
	}

	public static Norm createNorm(PBTerm normTerm) {
		PBTerm idTerm = normTerm.getArgument(1);
		String normSystemID = idTerm.getArgument(1).toString();
		String normID = idTerm.getArgument(2).toString();
		// TODO: handle non-elementary norms!
		ElementaryGround ground = Ground.createGround(normTerm.getArgument(2));
		ElementaryConsequence consequence = Consequence.createConsequence(normTerm.getArgument(3));
		return new ElementaryNorm(normSystemID,normID,ground,consequence);
	}
}
