package se.hig.jdalmas.norms;

import se.hig.jdalmas.GenericComponent;
import se.hig.jdalmas.exceptions.DnrDALMASException;
import se.sics.prologbeans.PBTerm;

public class Operator extends GenericComponent {
	protected String opID;
	protected String parameter;
	protected int arity;

	/**
	 * @param opID
	 */
	public Operator(String opID) {
		this.opID = opID;
		this.arity = 0;
	}

	/**
	 * @param opID
	 * @param parameter
	 */
	public Operator(String opID, String parameter) {
		this.opID = opID;
		this.arity = 1;
		this.parameter = parameter;
	}

	/**
	 * @return the opID
	 */
	public String getOpID() {
		return opID;
	}

	/**
	 * @return the arity
	 */
	public int getArity() {
		return arity;
	}

	/**
	 * @return the parameter
	 */
	public String getParameter() {
		if (this.arity < 1)
			throw new DnrDALMASException("Error: operator " + this.opID + " has no parameters");
		return this.parameter;
	}
	
	public static Operator createOperator(String op, String opParameter) {
		if (op == null || op.equals(""))
			throw new DnrDALMASException(op + "(" + opParameter + "): invalid operator");
		if (op.equals(MOVE))
			if (opParameter == null)
				return new MoveOperator();
			else
				return new MoveOperator(opParameter);
		if (op.equals(TYPE))
			return new TypeOperator(opParameter);
		return new Operator(op,opParameter);
	}

	public static PBTerm createPrologTerm(Operator op) {
		if (op.getArity() == 0)
			return PBTerm.makeAtom(op.getOpID());
		PBTerm [] args = new PBTerm[1];
		try {
			int p = Integer.parseInt(op.getParameter());
			args[0] = PBTerm.makeTerm(p);
		} catch (NumberFormatException nfe) {
			args[0] = PBTerm.makeAtom(op.getParameter());
		}
		return PBTerm.makeTerm(op.getOpID(),args);
	}

	public static Operator createOperator(PBTerm opTerm) {
		if (opTerm.getName().equals(MOVE))
			return createMoveOperator(opTerm);
		if (opTerm.getName().equals(TYPE))
			return createTypeOperator(opTerm);
		if (opTerm.isCompound()) {
			PBTerm parameterTerm = opTerm.getArgument(1);
			return new Operator(opTerm.getName(),parameterTerm.toString());
		}
		return new Operator(opTerm.getName());
	}
	public static MoveOperator createMoveOperator(PBTerm opTerm) {
		if (opTerm.isCompound()) {
			PBTerm parameterTerm = opTerm.getArgument(1);
			String parameter = parameterTerm.toString();
			if (parameter.equals("1"))
				return MoveOperator.M[1];
			return new MoveOperator(parameter);
		}
		return MoveOperator.M[0];
	}

	public static TypeOperator createTypeOperator(PBTerm opTerm) {
		PBTerm parameterTerm = opTerm.getArgument(1);
		int i = (int)parameterTerm.intValue();
//		return TypeOperator.T[i-1];
		return new TypeOperator(i + "");
	}

}