package se.hig.jdalmas.server;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import se.hig.jdalmas.control.PrologSessionHandler;
import se.hig.jdalmas.exceptions.DnrDALMASException;


public class DnrDALMASServer {
	protected static final String LOCALHOST = "localhost";
	protected File sicstusPath = null;
	protected File serverFile = null;
	protected String serverStartupGoal = null;
	protected ServerThread serverThread = null;
	protected boolean serverThreadRunning = false;
	protected Process sicstusProcess = null;
	protected BufferedReader sicstusErrorStreamReader = null;
	protected BufferedReader sicstusOutputStreamReader = null;
	protected int timeout = 10000;

	/**
	 * @param sicstusPath
	 * @param serverFileName
	 * @param serverStartupGoal
	 * @param serverDirName
	 */
	public DnrDALMASServer(File sicstusPath, File serverFile, String serverStartupGoal) {
		this.sicstusPath = sicstusPath;
		this.serverFile = serverFile;
		this.serverStartupGoal = serverStartupGoal;
		this.serverThread = new ServerThread();
	}

	public void start() {
		Thread t = new Thread(serverThread);
		t.setDaemon(true);
		t.start();
	}

	public void shutdown() {
		System.out.println("# Shutting down server... ");
		try {
			if (serverThread.normalShutdown())
				System.out.println("# Server shutdown OK!");
		} catch(InterruptedException ie) {
			System.err.println("# Server daemon interrupted: " + ie.getMessage());
		}
		System.out.println("# Destroying server process... ");
		sicstusProcess.destroy();
		System.out.println("# Server process destroyed!");
	}


	public boolean isRunning() {
		return this.serverThreadRunning;
	}


	/**
	 * @return the cfServerPort
	 */
	public int getPort() {
		int port = -1;
		try {
			port = serverThread.getPort();
		} catch(InterruptedException ie) {
			System.err.println("# Server daemon interrupted: " + ie.getMessage());
		}
		return port;
	}

	/**
	 * @return the timeout
	 */
	public int getTimeout() {
		return timeout;
	}


	/**
	 * @param timeout the timeout to set
	 */
	public void setTimeout(int timeout) {
		if (timeout < 0)
			throw new DnrDALMASException("Error: timeout value must be positive");
		this.timeout = timeout;
	}


	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		if (this.serverThreadRunning)
			shutdown();
	}

	protected class ServerThread implements Runnable {
		private int serverPort = -1;

		public void run() {
			System.out.println("# Server daemon has started");
			serverThreadRunning = true;
			String startCommand = "\"" + sicstusPath + "\" -l \"" + serverFile + 
			"\" --goal " + serverStartupGoal + ".";
			System.out.println("# Starting server: " + startCommand + "");
			System.out.println("# Working directory: " + serverFile.getParent());
			try {
				sicstusProcess = 
					Runtime.getRuntime().exec(startCommand,null,serverFile.getParentFile());
				sicstusErrorStreamReader = new BufferedReader(
						new InputStreamReader(sicstusProcess.getErrorStream()));
				while(serverThreadRunning) {
					String errLine = sicstusErrorStreamReader.readLine();
					if (errLine != null)
						if(errLine.startsWith("port:")) {
							serverPort = Integer.parseInt(errLine.substring(5));
							System.out.println("# Server listening at port " + serverPort);
							synchronized(this) {
								notify();
							}
						}
						else
							System.err.println(errLine);
				}
			} catch (IOException ioe) {
				throw new DnrDALMASException(ioe.getMessage());
			}
			System.out.println("# Server daemon has finished");				
		}

		public synchronized int getPort() throws InterruptedException {
			if (serverPort<0)
				wait(timeout);
			return serverPort;
		}

		public synchronized boolean normalShutdown() throws InterruptedException {
			wait(timeout); // Give server thread some time to finish up
			serverThreadRunning = false;
			PrologSessionHandler.getInstance().createSession(LOCALHOST,getPort());
			return PrologSessionHandler.getInstance().shutdown();
		}
	}

}