/**
 * 
 */
package se.hig.jdalmas.server;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import se.hig.jdalmas.exceptions.DnrDALMASException;
import se.hig.jdalmas.gui.SpecificDALMASGUI;

/**
 * @author mbm
 *
 */
public class DnrDALMASServerGUI extends JFrame {
	public final int W = 60;
	public final int H = 20;
	public static final String startupGoal = "start_server(50009)";
//	protected String sicstusPathString = 
//		System.getProperty("se.sics.prologbeans.prolog","sicstus");
	protected File sicstusPath = 
		new File("C:/Program Files (x86)/SICStus Prolog VC9 4.2.0/bin/sicstus.exe");
//	protected File sicstusPath = 
//		new File(sicstusPathString);
	protected File serverFile = null;
	protected SpecificDALMASGUI localClient = null;
	protected DnrDALMASServer dnrDALMASServer = null;
	protected String host = "localhost";
	protected int port = -1;

	protected JPanel centerPanel = new JPanel();
	protected JPanel messagePanel = new JPanel();

	protected JPanel chooseSicstusPathPanel = new JPanel();
	protected JLabel chooseSicstusPathLabel = new JLabel();
	protected JButton chooseSicstusPathBtn = 
		new JButton("Choose SICStus executable...");
	protected JFileChooser sicstusPathChooser = new JFileChooser(sicstusPath);

	protected JPanel chooseServerFilePanel = new JPanel();
	protected JLabel chooseServerFileLabel = new JLabel();
	protected JButton chooseServerFileBtn = 
		new JButton("Choose server file...");
	protected JFileChooser serverFileChooser = new JFileChooser(serverFile);

	protected JPanel startupGoalPanel = new JPanel();
	protected JLabel startupGoalLabel = new JLabel("Server startup goal: " + startupGoal);

	protected JPanel timeoutPanel = new JPanel();
	protected JLabel timeoutLabel = new JLabel("Timeout value (ms):");
	protected JTextField timeoutTxt = new JTextField();
	protected JButton timeoutBtn = new JButton("Apply new timeout value");

	protected JPanel serverControlPanel = new JPanel();
	protected JButton startServerBtn = new JButton("Start Prolog server");
	protected JButton stopServerBtn = new JButton("Stop Prolog server");
	protected JLabel serverStartedLabel = new JLabel();

	protected JButton startLocalClientBtn = new JButton("Start local DALMAS client");


	public DnrDALMASServerGUI(String serverFileString) {
		this(serverFileString,null);
	}

	public DnrDALMASServerGUI(String serverFileString, SpecificDALMASGUI localClient) {
		if (serverFileString != null)
			this.serverFile = new File(serverFileString);
		this.localClient = localClient;
		initGUI();
	}

	private void initGUI() {
		this.setPreferredSize(new Dimension(10*W,16*H));
		setTitle("DnrDALMAS Server [" + serverFile + "]");

		// --- CENTER PANEL ---
		centerPanel.setLayout(new BoxLayout(centerPanel,BoxLayout.Y_AXIS));
		this.add(centerPanel,BorderLayout.CENTER);

		// --- CHOOSE SICSTUS EXECUTABLE PANEL ---
		sicstusPathChooser.setDialogTitle("Choose SICStus executable");
		chooseSicstusPathBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				int choice = sicstusPathChooser.showOpenDialog(centerPanel);
				if (choice == JFileChooser.APPROVE_OPTION)
					sicstusPath = sicstusPathChooser.getSelectedFile();
				chooseSicstusPathLabel.setText("Current executable: " + sicstusPath);
			}
		});
		chooseSicstusPathPanel.add(chooseSicstusPathBtn);
		chooseSicstusPathLabel.setText("Current executable: " + sicstusPath);
		chooseSicstusPathPanel.add(chooseSicstusPathLabel);
		centerPanel.add(chooseSicstusPathPanel);

		// --- CHOOSE SERVER DIR PANEL ---
		serverFileChooser.setDialogTitle("Choose server file");
		chooseServerFileBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				int choice = serverFileChooser.showOpenDialog(centerPanel);
				if (choice == JFileChooser.APPROVE_OPTION) {
					serverFile = serverFileChooser.getSelectedFile();
					setTitle("DnrDALMAS Server [" + serverFile.getName() + "]");
					chooseServerFileLabel.setText("Current working dir: " + 
							serverFile.getParent());
				}
			}
		});
		chooseServerFilePanel.add(chooseServerFileBtn);
		chooseServerFileLabel.setText("Current working dir: " + 
				(serverFile == null ? "-" : serverFile.getParent()));
		chooseServerFilePanel.add(chooseServerFileLabel);
		centerPanel.add(chooseServerFilePanel);

		// --- STARTUP GOAL PANEL ---
		startupGoalPanel.add(startupGoalLabel);
		centerPanel.add(startupGoalPanel);

		// --- TIMEOUT PANEL ---
		timeoutPanel.add(timeoutLabel);
		timeoutTxt.setPreferredSize(new Dimension(W,H));
		timeoutTxt.setText("8000");
		timeoutBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				applyNewTimeoutValue();
			}
		});
		timeoutPanel.add(timeoutTxt);
		timeoutPanel.add(timeoutBtn);
		centerPanel.add(timeoutPanel);

		// --- SERVER CONTROL PANEL ---
		startServerBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if ((dnrDALMASServer != null) && dnrDALMASServer.isRunning())
					JOptionPane.showMessageDialog(centerPanel, 
							"The server is already running", 
							"Note",JOptionPane.INFORMATION_MESSAGE);
				else if (serverFile == null)
					JOptionPane.showMessageDialog(centerPanel,
							"Please choose a dnrDALMAS server file", 
							"Note",JOptionPane.INFORMATION_MESSAGE);
				else {
					startServer();
					chooseSicstusPathBtn.setEnabled(false);
					chooseServerFileBtn.setEnabled(false);
				}
			}
		});
		serverControlPanel.add(startServerBtn);

		stopServerBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if ((dnrDALMASServer == null) || !dnrDALMASServer.isRunning())
					JOptionPane.showMessageDialog(centerPanel, 
							"The server is not running", 
							"Note",JOptionPane.INFORMATION_MESSAGE);
				else {
					shutdownServer();
					chooseSicstusPathBtn.setEnabled(true);
					chooseServerFileBtn.setEnabled(true);
				}
			}
		});
		serverControlPanel.add(stopServerBtn);

		startLocalClientBtn.setEnabled(false);
		startLocalClientBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if (localClient != null && dnrDALMASServer.isRunning())
					startLocalClient();
				else {
					JOptionPane.showMessageDialog(centerPanel, 
							"Local client not available", 
							"Note", JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		serverControlPanel.add(startLocalClientBtn);
		centerPanel.add(serverControlPanel);

		// --- MESSAGE PANEL ---
		messagePanel.add(serverStartedLabel);
		this.add(messagePanel,BorderLayout.SOUTH);

		// --- MAIN FRAME ---
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				if ((dnrDALMASServer != null) && dnrDALMASServer.isRunning())
					shutdownServer();
				System.exit(0);
			}
		});
		this.pack();
	}

	public void applyNewTimeoutValue() {
		try {
			int timeout = Integer.parseInt(timeoutTxt.getText());
			dnrDALMASServer.setTimeout(timeout);
		} catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(centerPanel, 
					"Timeout value must be an integer value", 
					"Failure", JOptionPane.ERROR_MESSAGE);
		} catch (DnrDALMASException de) {
			JOptionPane.showMessageDialog(centerPanel,de.getMessage(), 
					"Failure",JOptionPane.ERROR_MESSAGE);
		}
	}

	private void startServer() {
		dnrDALMASServer = 
			new DnrDALMASServer(sicstusPath,serverFile,startupGoal);
		applyNewTimeoutValue();
		dnrDALMASServer.start();
		port = dnrDALMASServer.getPort();
		if (port < 0)
			JOptionPane.showMessageDialog(this,"The server could not be started", 
					"Failure",JOptionPane.ERROR_MESSAGE);
		else
		{
			serverStartedLabel.setText("Server listening at port " + port);
			startLocalClientBtn.setEnabled(true);
		}			
	}

	private void shutdownServer () {
		startLocalClientBtn.setEnabled(false);
		serverStartedLabel.setText("Terminating server... ");
		dnrDALMASServer.shutdown();
		port = -1;
		serverStartedLabel.setText("Server terminated");
	}

	private void startLocalClient() {
		if (localClient != null)
			try {
				localClient.startClient(host,port);
			} catch (DnrDALMASException de) {
				JOptionPane.showMessageDialog(centerPanel,de.getMessage(), 
						"Failure",JOptionPane.ERROR_MESSAGE);					
			}
			else
				JOptionPane.showMessageDialog(centerPanel,"Local client not available", 
						"Note",JOptionPane.INFORMATION_MESSAGE);			
	}

	public static void main(String [] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				DnrDALMASServerGUI serverGUI = new DnrDALMASServerGUI(null);			
				serverGUI.setVisible(true);
			}
		});
	}

}
