/**
 * 
 */
package se.hig.wasteland.client;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import se.hig.jdalmas.Predicate;
import se.hig.jdalmas.exceptions.DnrDALMASException;
import se.hig.jdalmas.gui.JDALMASGUI;
import se.hig.jdalmas.gui.SpecificDALMASGUI;
import se.hig.jdalmas.gui.StepNextSituationFrame;

import java.awt.HeadlessException;
import javax.swing.BorderFactory;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.GridLayout;
import java.util.Observable;

/**
 * @author mbm
 *
 */
public class WasteDALMASGUI extends JFrame implements SpecificDALMASGUI {
	private WastelandFrame wastelandFrame;
	private WasteDALMASController wasteDALMASController;
	private JDALMASGUI jDALMASGUI = new JDALMASGUI();
	private Box centerBox;
	private Box southBox;
	private JButton updateKnowledgeBtn;
	private JButton stepNextSituationBtn;
	private JButton nextSituationBtn;
	private JButton kEventRunBtn;

	private void initGUI() {
		centerBox = Box.createVerticalBox();
		southBox = Box.createHorizontalBox();
		updateKnowledgeBtn = new JButton("Update knowledge base...");
		stepNextSituationBtn = new JButton("Step >");
		nextSituationBtn = new JButton("Next situation >>");
		kEventRunBtn = new JButton("K-event run >>>");
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				stopClient();
			}
		});
		updateKnowledgeBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				addKnowledge();
			}
		});
		stepNextSituationBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				StepNextSituationFrame stepFrame = 
					new StepNextSituationFrame(jDALMASGUI);
				stepFrame.setVisible(true);
			}
		});
		nextSituationBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				try {
					jDALMASGUI.getDnrDALMASController().nextSituation();
//					displaySituation();
				} catch (DnrDALMASException de) {
					JOptionPane.showMessageDialog(WasteDALMASGUI.this,de.getMessage(), 
							"Note",JOptionPane.INFORMATION_MESSAGE);
				}
			}
		});
		kEventRunBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				try {
					jDALMASGUI.getDnrDALMASController().kEventRun(4);
//					displayRun();
				} catch (DnrDALMASException de) {
					JOptionPane.showMessageDialog(WasteDALMASGUI.this,de.getMessage(), 
					"Failure",JOptionPane.ERROR_MESSAGE);					
				}
			}
		});
		centerBox.add(jDALMASGUI);
		southBox.add(updateKnowledgeBtn);
		southBox.add(stepNextSituationBtn);
		southBox.add(nextSituationBtn);
		southBox.add(kEventRunBtn);
		centerBox.add(southBox);
		this.add(centerBox);
		this.setVisible(true);
		this.pack();
	}
		
	public void initDnrDALMAS() {
		jDALMASGUI.addAgent("x3");
		jDALMASGUI.addAgent("x2");
		jDALMASGUI.addAgent("x1");
		jDALMASGUI.addAct("pass");
		jDALMASGUI.addAct("go","east");
		jDALMASGUI.addAct("go","north");
		jDALMASGUI.addAct("go","west");
		jDALMASGUI.addAct("go","south");
//		jDALMASGUI.addPredicate(Predicate.ACT_RESULT_PREDICATE,
//				"user","agentResultOfAct",4);
		jDALMASGUI.addPredicate(Predicate.FEASIBLE_ACT_PREDICATE, 
				"user","agentFeasibleAct",3);
		jDALMASGUI.addPredicate(Predicate.UTILITY_FUNCTION_PREDICATE,
				"user","agentUtilityFunction",4);
		jDALMASGUI.addGroundToGroundSet("diff","user","diff",0,2);
		jDALMASGUI.addConsequenceToConsequenceSet("lap","user","overlap",1,2);
		jDALMASGUI.addElementaryNorm("n1",1,"diff",null,7,"lap","9");
	}
	
	public void initKnowledge() {
		wasteDALMASController.addBoundaries(0,0,7,7);
		wasteDALMASController.addWaste();
		wasteDALMASController.addAgentPosition("x3",2,1);
		wasteDALMASController.addAgentPosition("x2",1,2);
		wasteDALMASController.addAgentPosition("x1",1,1);
		wasteDALMASController.addAgentWasteLoad("x3",0);
		wasteDALMASController.addAgentWasteLoad("x2",0);
		wasteDALMASController.addAgentWasteLoad("x1",0);
		wasteDALMASController.addPreviousAct("x3","pass",null);
		wasteDALMASController.addPreviousAct("x2","pass",null);
		wasteDALMASController.addPreviousAct("x1","pass",null);
		wastelandFrame = new WastelandFrame();
	}
	
	protected void addKnowledge() {
		UpdateKnowledgeFrame addKnowledgeFrame = new UpdateKnowledgeFrame(this);
		addKnowledgeFrame.setVisible(true);
	}

	public void addAgentPosition(String agentID, int x, int y)
	{
		if (jDALMASGUI.getDnrDALMASController().containsAgent(agentID)) {
			wasteDALMASController.addAgentPosition(agentID,x,y);
//			displayKnowledge();
		}
		else
			JOptionPane.showMessageDialog(wastelandFrame,agentID + ": no such agent", 
					"Error",JOptionPane.ERROR_MESSAGE);
	}
	
	public void updateAgentPosition(String agentID, int x, int y)
	{
		if (jDALMASGUI.getDnrDALMASController().containsAgent(agentID)) {
			wasteDALMASController.updateAgentPosition(agentID,x,y);
//			displayKnowledge();
		}
		else
			JOptionPane.showMessageDialog(wastelandFrame,agentID + ": no such agent", 
					"Error",JOptionPane.ERROR_MESSAGE);
	}
	
	public void addWall(String x, String y) {
		wasteDALMASController.addWallPosition(x,y);
	}
	
	private void displayRun() {
//		jDALMASGUI.displayRun();
		displaySituation();
	}

	private void displaySituation() {
//		jDALMASGUI.displaySituation();
		displayKnowledge();
	}

	private void displayKnowledge() {
		wasteDALMASController.refreshWastelandKB();
		wastelandFrame.display();
	}
		
	public void update(Observable arg0, Object arg1) {
		displaySituation();
	}

	public void startClient(String host, int port) {
		jDALMASGUI.createController(host,port);
		jDALMASGUI.setVisible(true);
		wasteDALMASController = new WasteDALMASController(jDALMASGUI.getDnrDALMASController());
		this.setTitle("Wasteland Client @ " + host + ":" + port);
		this.initGUI();
		this.initDnrDALMAS();
		this.initKnowledge();
		jDALMASGUI.getDnrDALMASController().addObserver(this);
		this.displaySituation();
	}

	public void stopClient() {
		wastelandFrame.dispose();
		this.getContentPane().removeAll();
		this.dispose();
	}

	/**
	 * @author mbm
	 *
	 */
	private class WastelandFrame extends JFrame {
		private final int SQUARE_SIZE = 80;
		private Box centerBox = Box.createVerticalBox();
		private JPanel mainPanel = new JPanel();
		private Box southBox = Box.createVerticalBox();
		private JLabel uncollectedWasteLabel = new JLabel();
		private JLabel collectedWasteLabel = new JLabel();
		private SquareButton[][] grid = null;
	
		/**
		 * @throws HeadlessException
		 */
		public WastelandFrame() throws HeadlessException {
			super("Wasteland");
			initFrame();
		}
	
		private void initFrame() {
			int xMin = wasteDALMASController.getXMin();
			int xMax = wasteDALMASController.getXMax();
			int yMin = wasteDALMASController.getYMin();
			int yMax = wasteDALMASController.getYMax();
			int w = xMax - xMin - 1;
			int h = yMax - yMin - 1;
			mainPanel.setLayout(new GridLayout(w, h));
			grid = new SquareButton[w][h];
			for (int y = yMax - 1; y > yMin; y--)
				for (int x = xMin + 1; x < xMax; x++) {
					int i = x - xMin - 1;
					int j = y - yMin - 1;
					grid[i][j] = new SquareButton(x, y);
					grid[i][j].setPreferredSize(new Dimension(SQUARE_SIZE,
							SQUARE_SIZE));
					grid[i][j].setForeground(Color.YELLOW);
					mainPanel.add(grid[i][j]);
				}
			centerBox.add(mainPanel);
			southBox.add(uncollectedWasteLabel);
			southBox.add(collectedWasteLabel);
			centerBox.add(southBox);
			this.add(centerBox);
			this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			this.pack();
			this.setVisible(true);
		}
	
		private void display() {
			wasteDALMASController.updateGrid();
			int xMin = wasteDALMASController.getXMin();
			int xMax = wasteDALMASController.getXMax();
			int yMin = wasteDALMASController.getYMin();
			int yMax = wasteDALMASController.getYMax();
	
			for (int x = xMin + 1; x < xMax; x++)
				for (int y = yMin + 1; y < yMax; y++) {
					int i = x - xMin - 1;
					int j = y - yMin - 1;
					float c = wasteDALMASController.getAmountOfWaste(x,y)
							/ (float)wasteDALMASController.MAX_WASTE_PER_SQUARE;
					grid[i][j].setBackground(new Color(c, c, c));
					grid[i][j].setBorder(BorderFactory
							.createLineBorder(Color.DARK_GRAY));
					AgentState agentState = wasteDALMASController.getAgentAtSquare(x,y);
					if (agentState == null)
						grid[i][j].setText("");
					else {
						grid[i][j].setText(agentState.getAgentID());
					}
				}
			String agentToMove = jDALMASGUI.getDnrDALMASController().getAgentToMove();
			int i = wasteDALMASController.getAgentX(agentToMove) - xMin - 1;
			int j = wasteDALMASController.getAgentY(agentToMove) - yMin - 1;
			grid[i][j].setBorder(BorderFactory.createLineBorder(Color.GREEN));
			uncollectedWasteLabel.setText("Amount of uncollected waste: " + 
					wasteDALMASController.getUncollectedWaste());
			collectedWasteLabel.setText("Amount of collected waste: " + 
					wasteDALMASController.getCollectedWaste());
		}
	
		class SquareButton extends JButton {
			private int x;
			private int y;
	
			SquareButton(int x, int y) {
				this.x = x;
				this.y = y;
				this.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						String pos = "pos(" + SquareButton.this.x + ","
								+ SquareButton.this.y + ")";
						JOptionPane.showMessageDialog(SquareButton.this,
								SquareButton.this.toString(), pos,
								JOptionPane.PLAIN_MESSAGE);
					}
				});
			}
	
			/* (non-Javadoc)
			 * @see java.awt.Component#toString()
			 */
			@Override
			public String toString() {
				String squareString = "Amount of waste: "
						+ wasteDALMASController.getAmountOfWaste(x, y);
				AgentState agentState = wasteDALMASController.getAgentAtSquare(x,y);
				if (agentState == null)
					return squareString;
				return squareString + "\nAgent: " + agentState.toString();
			}
	
		}
	
	}

	/**
	 * @author mbm
	 *
	 */
	private class UpdateKnowledgeFrame extends JFrame {
		private JLabel agentIDLabel = new JLabel("Agent ID");
	
		private JPanel centerPanel = new JPanel();
	
		private JComboBox agentBox = null;
	
		private JButton chooseAgentBtn = new JButton("Choose agent");
		private JButton updateAgentStateBtn = new JButton("Update agent state");
	
		public UpdateKnowledgeFrame(WasteDALMASGUI parent) {
			super("Add agent");
			initFrame();
		}
	
		private void initFrame() {
			this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			this.setLayout(new BorderLayout());
			// --- CENTER PANEL ---
			centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
			centerPanel.add(agentIDLabel);
			agentBox = new JComboBox(jDALMASGUI.getAgentIDs());
			centerPanel.add(agentBox);
			centerPanel.add(chooseAgentBtn);
	
			// --- MAIN FRAME ---		
			this.add(centerPanel, BorderLayout.CENTER);
			chooseAgentBtn.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					UpdateKnowledgeFrame.this.add(updateAgentStateBtn,
							BorderLayout.SOUTH);
					UpdateKnowledgeFrame.this.agentBox.setEnabled(false);
					UpdateKnowledgeFrame.this.chooseAgentBtn.setEnabled(false);
					UpdateKnowledgeFrame.this.pack();
				}
			});
			updateAgentStateBtn.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					String agentID = agentBox.getSelectedItem().toString();
				}
			});
			this.pack();
		}
	
	}

}
