package se.hig.wasteland.evolutionary;

import se.hig.jdalmas.Knowledge;
import se.hig.jdalmas.KnowledgeBase;
import se.hig.jdalmas.ListKB;
import se.hig.jdalmas.control.DnrDALMASController;
import se.hig.jdalmas.exceptions.DnrDALMASException;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

public class WasteDALMASController {
	private int xMin;
	private int yMin;
	private int xMax;
	private int yMax;
	private int uncollectedWaste;
	private int collectedWaste;

	private WastelandSquare [][] wastelandGrid;
	private DnrDALMASController dnrDALMASController;
	private List<Knowledge> currentListKB = ListKB.createEmptyListKB();

	public final int MAX_WASTE_PER_SQUARE = 10;

	public WasteDALMASController(DnrDALMASController dnrDALMASController) {
		this.dnrDALMASController = dnrDALMASController;
	}

	/**
	 * @return the xMin
	 */
	protected int getXMin() {
		return xMin;
	}

	/**
	 * @return the yMin
	 */
	protected int getYMin() {
		return yMin;
	}

	/**
	 * @return the xMax
	 */
	protected int getXMax() {
		return xMax;
	}

	/**
	 * @return the yMax
	 */
	protected int getYMax() {
		return yMax;
	}

	/**
	 * @return the uncollectedWaste
	 */
	protected int getUncollectedWaste() {
		return uncollectedWaste;
	}

	/**
	 * @return the collectedWaste
	 */
	protected int getCollectedWaste() {
		System.err.println("Collected waste: " + collectedWaste);
		return collectedWaste;
	}

	public void addBoundaries(int xMin, int yMin, int xMax, int yMax) {
		this.xMin = xMin;
		this.yMin = yMin;
		this.xMax = xMax;
		this.yMax = yMax;
		int w = xMax-xMin-1;
		int h = yMax-yMin-1;
		this.wastelandGrid = new WastelandSquare[w][h];
		for (int x=0; x<w; x++)
			for(int y=0; y<h; y++)
				this.wastelandGrid[x][y] = new WastelandSquare(xMin+x,yMin+y);
		addWallPosition(""+xMin,"all");
		addWallPosition(""+xMax,"all");
		addWallPosition("all",""+yMin);
		addWallPosition("all",""+yMax);
	}

	public void addWallPosition(String x, String y) {
		Knowledge wallPosition = WastelandKB.createWallPosition(x,y);
		ListKB.addKnowledge(currentListKB,wallPosition);
		KnowledgeBase knowledgeBase = ListKB.createKnowledgeBase(currentListKB);
		dnrDALMASController.replaceCurrentKnowledgeBase(knowledgeBase);
	}

	public void addAgent(String agentID) {
		if (agentID == null || agentID.equals(""))
			throw new DnrDALMASException("Agent ID must not be empty");
		if (dnrDALMASController.containsAgent(agentID))
			throw new DnrDALMASException("Agent " + agentID + " already exists");
		dnrDALMASController.addAgent(agentID);
	}

	public void addAct(String actID) {
		addAct(actID,null);
	}

	public void addAct(String actID, String actParamsString) {
		if (actID == null || actID.equals(""))
			throw new DnrDALMASException("Act ID must not be empty");
		String [] actParams = null;
		if(actParamsString!=null && !actParamsString.equals(""))
			actParams = actParamsString.split(",");
		if (dnrDALMASController.containsAct(actID,actParams))
			throw new DnrDALMASException("Act " + actID + "(" + 
					actParamsString + ") already exists");
		dnrDALMASController.addAct(actID,actParams);
	}

	public void addAgentPosition(String agentID, int x, int y) {
		Knowledge agentPosition = WastelandKB.createAgentPosition(agentID,x,y);
		ListKB.addKnowledge(currentListKB,agentPosition);
		KnowledgeBase knowledgeBase = ListKB.createKnowledgeBase(currentListKB);
		dnrDALMASController.replaceCurrentKnowledgeBase(knowledgeBase);
	}

	public void updateAgentPosition(String agentID, int x, int y) {
		Knowledge oldAgentPosition = WastelandKB.getAgentPosition(currentListKB,agentID);
		ListKB.deleteKnowledge(currentListKB,oldAgentPosition);
		addAgentPosition(agentID,x,y);
	}

	public void addAgentWasteLoad(String agentID, int amountOfWaste) {
		Knowledge agentWasteLoad = 
			WastelandKB.createAgentWasteLoad(agentID,amountOfWaste);
		ListKB.addKnowledge(currentListKB,agentWasteLoad);
		KnowledgeBase knowledgeBase = ListKB.createKnowledgeBase(currentListKB);
		dnrDALMASController.replaceCurrentKnowledgeBase(knowledgeBase);		
	}

	public void addRandomWaste() {
		Random rnd = new Random();
		for (int x=xMin+1; x<xMax; x++)
			for(int y=yMin+1; y<yMax; y++)
				this.addWastePosition(rnd.nextInt(MAX_WASTE_PER_SQUARE),x,y);
	}

	public void addFixedWaste() {
		for (int x=xMin+1; x<xMax; x++)
			for(int y=yMin+1; y<yMax; y++)
				if(x == (xMin+1) && y == (yMin+1))
					this.addWastePosition(0,x,y);
				else
					this.addWastePosition(1,x,y);
	}

	public void addWastePosition(int amount, int x, int y) {
		Knowledge wastePosition = 
			WastelandKB.createWastePosition(amount,x,y);
		ListKB.addKnowledge(currentListKB,wastePosition);		
		KnowledgeBase knowledgeBase = ListKB.createKnowledgeBase(currentListKB);
		dnrDALMASController.replaceCurrentKnowledgeBase(knowledgeBase);		
	}

	public void addPreviousAct(String agentID, String actID, String actParameter) {
		Knowledge previousAct = 
			WastelandKB.createPreviousAct(agentID,actID,actParameter);
		ListKB.addKnowledge(currentListKB,previousAct);
		KnowledgeBase knowledgeBase = ListKB.createKnowledgeBase(currentListKB);
		dnrDALMASController.replaceCurrentKnowledgeBase(knowledgeBase);		
	}

	public int getAgentX(String agentID) {
		Knowledge agentPosition = WastelandKB.getAgentPosition(currentListKB,agentID);
		return WastelandKB.getX(agentPosition);
	}

	public int getAgentY(String agentID) {
		Knowledge agentPosition = WastelandKB.getAgentPosition(currentListKB,agentID);
		return WastelandKB.getY(agentPosition);
	}

	public AgentState getAgentAtSquare(int x, int y) {
		return wastelandGrid[x-1-xMin][y-1-yMin].getAgentState();
	}

	public int getAgentWasteLoad(String agentID) {
		Knowledge wasteLoad = WastelandKB.getAgentWasteLoad(currentListKB,agentID);
		return WastelandKB.getWasteLoad(wasteLoad);
	}

	public int getAmountOfWaste(int x, int y) {
		return wastelandGrid[x-1-xMin][y-1-yMin].getAmountOfWaste();
	}

	public void updateGrid() {
		Map<String,AgentState> agentStateMap = new HashMap<String,AgentState>();
		uncollectedWaste = 0;
		collectedWaste = 0;
		for (Knowledge knowledge : currentListKB) {
			if (WastelandKB.isWastePosition(knowledge)) {
				int x = WastelandKB.getX(knowledge)-1-xMin;
				int y = WastelandKB.getY(knowledge)-1-yMin;
				wastelandGrid[x][y].setAgentState(null);
				int amountOfWaste = WastelandKB.getAmountOfWaste(knowledge);
				wastelandGrid[x][y].setAmountOfWaste(amountOfWaste);
				uncollectedWaste += amountOfWaste;
			} else if (WastelandKB.isAgentPosition(knowledge)) {
				int x = WastelandKB.getX(knowledge)-1-xMin;
				int y = WastelandKB.getY(knowledge)-1-yMin;
				String agentID = WastelandKB.getAgentID(knowledge);
				if (!agentStateMap.containsKey(agentID))
					agentStateMap.put(agentID,new AgentState(agentID));
				AgentState agentState = agentStateMap.get(agentID);
				agentState.setGridX(x);
				agentState.setGridY(y);
			} else if (WastelandKB.isAgentWasteLoad(knowledge)) {
				String agentID = WastelandKB.getAgentID(knowledge);
				if (!agentStateMap.containsKey(agentID))
					agentStateMap.put(agentID,new AgentState(agentID));
				int wasteLoad = WastelandKB.getWasteLoad(knowledge);
				agentStateMap.get(agentID).setWasteLoad(wasteLoad);
				collectedWaste += wasteLoad;
			}
		}
		for (AgentState agentState : agentStateMap.values()) {
			int x = agentState.getGridX();
			int y = agentState.getGridY();
			wastelandGrid[x][y].setAgentState(agentState);
		}
	}

	public void refreshWastelandKB() {
		currentListKB = ListKB.createListKB(dnrDALMASController.getCurrentKnowledgeBase());
	}

	public DnrDALMASController getDnrDALMASController() {
		return dnrDALMASController;
	}

//	public void nextSituation() throws DnrDALMASException {
//		for (String agentID: dnrDALMASController.getAgentSet())
//			if (WastelandKB.getAgentPosition(currentKB,agentID) == null)
//				throw new DnrDALMASException("Agent " + agentID + " has no position");
//		// TODO: add corresponding code for previous action and holding waste
//		dnrDALMASController.nextSituation();
//		currentKB = ListKB.createListKB(dnrDALMASController.getCurrentKnowledgeBase());
//	}

}