/**
 * 
 */
package se.hig.wasteland.evolutionary;

import java.util.LinkedList;
import java.util.List;

import se.hig.jdalmas.Knowledge;
import se.sics.prologbeans.PBTerm;

/**
 * @author mbm
 *
 */
public class WastelandKB {
	public static final String AT = "at";
	public static final String POS = "pos";
	public static final String WASTE = "waste";
	public static final String WALL = "wall";
	public static final String ALL = "all";
	public static final String HOLDING = "holding";
	public static final String JUMP = "jump";
	public static final String PREVIOUS_ACT = "previousAction";

	public static PBTerm createPosTerm(int x, int y) {
		PBTerm [] arguments = {
				PBTerm.makeTerm(x),
				PBTerm.makeTerm(y)
		};
		return PBTerm.makeTerm(POS,arguments);
	}

	public static PBTerm createAgentPositionTerm(String agentID, int x, int y) {
		PBTerm posTerm = createPosTerm(x,y);
		PBTerm [] arguments = { 
				PBTerm.makeAtom(agentID),
				posTerm
		};
		return PBTerm.makeTerm(AT,arguments);
	}

	public static Knowledge createAgentPosition(String agentID, int x, int y) {
		return Knowledge.createKnowledge(createAgentPositionTerm(agentID,x,y));
	}

	public static Knowledge getAgentPosition(List<Knowledge> listKB, String agentID) {
		for (Knowledge knowledge : listKB) {
			if (isAgentPosition(knowledge))
				if (agentID.equals(getAgentID(knowledge)))
					return knowledge;
		}
		return null;
	}

	public static boolean isAgentPosition(Knowledge knowledge) {
		PBTerm knowledgeTerm = knowledge.getPrologTerm();
		if (AT.equals(knowledgeTerm.getName())) {
			String objectID = knowledgeTerm.getArgument(1).getName();
			if (WASTE.equals(objectID))
				return false;
			if (WALL.equals(objectID))
				return false;
			return true;
		}
		return false;
	}
	
	public static String getAgentID(Knowledge knowledge) {
		PBTerm prologTerm = knowledge.getPrologTerm();
		return prologTerm.getArgument(1).toString();
	}

	public static int getX(Knowledge knowledge) {
		PBTerm prologTerm = knowledge.getPrologTerm();
		PBTerm posTerm = prologTerm.getArgument(2);
		return (int)posTerm.getArgument(1).intValue();
	}
	
	public static int getY(Knowledge knowledge) {
		PBTerm prologTerm = knowledge.getPrologTerm();
		PBTerm posTerm = prologTerm.getArgument(2);
		return (int)posTerm.getArgument(2).intValue();
	}
	
	public static List<Knowledge> getAgentPositions(List<Knowledge> listKB) {
		LinkedList<Knowledge> agentPositions = new LinkedList<Knowledge>();
		for (Knowledge knowledge : listKB) {
			if (isAgentPosition(knowledge)) {
				agentPositions.add(knowledge);				
			}
		}
		return agentPositions;
	}

	public static PBTerm createAgentWasteLoadTerm(String agentID, int wasteLoad) {
		PBTerm [] wasteAmountTerm = { PBTerm.makeTerm(wasteLoad) };
		PBTerm wasteTerm = PBTerm.makeTerm(WASTE,wasteAmountTerm);
		PBTerm [] arguments = { 
				PBTerm.makeAtom(agentID), 
				wasteTerm 
		};
		return PBTerm.makeTerm(HOLDING,arguments);
	}

	public static Knowledge createAgentWasteLoad(String agentID, int wasteLoad) {
		return Knowledge.createKnowledge(createAgentWasteLoadTerm(agentID,wasteLoad));
	}

	public static Knowledge getAgentWasteLoad(List<Knowledge> listKB, String agentID) {
		for (Knowledge knowledge : listKB) {
			if (isAgentWasteLoad(knowledge))
				if (agentID.equals(getAgentID(knowledge)))
					return knowledge;
		}
		return null;
	}

	public static boolean isAgentWasteLoad(Knowledge knowledge) {
		PBTerm knowledgeTerm = knowledge.getPrologTerm();
		if (HOLDING.equals(knowledgeTerm.getName())) {
			return true;
		}
		return false;
	}
	
	public static int getWasteLoad(Knowledge knowledge) {
		PBTerm prologTerm = knowledge.getPrologTerm();
		PBTerm loadTerm = prologTerm.getArgument(2);
		return (int)loadTerm.getArgument(1).intValue();
	}
	
	public static List<Knowledge> getAgentWasteLoads(List<Knowledge> listKB) {
		LinkedList<Knowledge> agentWasteLoads = new LinkedList<Knowledge>();
		for (Knowledge knowledge : listKB) {
			if (isAgentWasteLoad(knowledge)) {
				agentWasteLoads.add(knowledge);				
			}
		}
		return agentWasteLoads;
	}

	public static PBTerm createPreviousActTerm(String agentID, String actID, String actParameter) {
		PBTerm actTerm = null;
		if (actParameter == null || actParameter.equals(""))
			actTerm = PBTerm.makeAtom(actID);
		else {
			PBTerm [] parameterTerm = { PBTerm.makeAtom(actParameter) };
			actTerm = PBTerm.makeTerm(actID,parameterTerm);
		}
		PBTerm [] arguments = { 
				PBTerm.makeAtom(agentID),
				actTerm
		};
		return PBTerm.makeTerm(PREVIOUS_ACT,arguments);
	}

	public static Knowledge 
	createPreviousAct(String agentID, String actID, String actParameter) {
		return Knowledge.createKnowledge(createPreviousActTerm(agentID,actID,actParameter));
	}

	public static Knowledge getPreviousAct(List<Knowledge> listKB, String agentID) {
		for (Knowledge knowledge : listKB) {
			if (isPreviousAct(knowledge))
				if (agentID.equals(getAgentID(knowledge)))
					return knowledge;
		}
		return null;
	}

	public static boolean isPreviousAct(Knowledge knowledge) {
		PBTerm knowledgeTerm = knowledge.getPrologTerm();
		if (PREVIOUS_ACT.equals(knowledgeTerm.getName())) {
			return true;
		}
		return false;
	}
	
	public static String getPreviousActString(Knowledge knowledge) {
		PBTerm prologTerm = knowledge.getPrologTerm();
		PBTerm previousActTerm = prologTerm.getArgument(2);
		return previousActTerm.toString();
	}
	
	public static List<Knowledge> getPreviousActs(List<Knowledge> listKB) {
		LinkedList<Knowledge> previousActs = new LinkedList<Knowledge>();
		for (Knowledge knowledge : listKB) {
			if (isPreviousAct(knowledge)) {
				previousActs.add(knowledge);				
			}
		}
		return previousActs;
	}

	public static PBTerm createPosTerm(String x, String y) {
		PBTerm [] arguments = { 
				(ALL.equals(x) ? PBTerm.makeAtom(ALL) : PBTerm.makeTerm(Integer.parseInt(x))),
				(ALL.equals(y) ? PBTerm.makeAtom(ALL) : PBTerm.makeTerm(Integer.parseInt(y)))
		};
		return PBTerm.makeTerm(POS,arguments);
	}
	
	public static PBTerm createWallPositionTerm(String x, String y) {
		PBTerm posTerm = createPosTerm(x,y);
		PBTerm [] arguments = {
				PBTerm.makeAtom(WALL),
				posTerm
		};
		return PBTerm.makeTerm(AT,arguments);
	}
	
	public static Knowledge createWallPosition(String x, String y) {
		return Knowledge.createKnowledge(createWallPositionTerm(x,y));
	}

	public static boolean isWallPosition(Knowledge knowledge) {
		PBTerm knowledgeTerm = knowledge.getPrologTerm();
		if (AT.equals(knowledgeTerm.getName())) {
			String objectID = knowledgeTerm.getArgument(1).getName();
			if (WALL.equals(objectID))
				return true;
		}
		return false;
	}
	
	public static List<Knowledge> getWallPositions(List<Knowledge> listKB) {
		LinkedList<Knowledge> wallPositions = new LinkedList<Knowledge>();
		for (Knowledge knowledge : listKB) {
			if (isWallPosition(knowledge)) {
				wallPositions.add(knowledge);				
			}
		}
		return wallPositions;
	}

	public static PBTerm createWastePositionTerm(int amountOfWaste, int x, int y) {
		PBTerm [] wasteAmountTerm = {PBTerm.makeTerm(amountOfWaste)};
		PBTerm wasteTerm = PBTerm.makeTerm(WASTE,wasteAmountTerm);
		PBTerm posTerm = createPosTerm(x,y);
		PBTerm [] arguments = {wasteTerm,posTerm};
		return PBTerm.makeTerm(AT,arguments);
	}
	
	public static Knowledge createWastePosition(int amountOfWaste, int x, int y) {
		return Knowledge.createKnowledge(createWastePositionTerm(amountOfWaste,x,y));
	}

	public static boolean isWastePosition(Knowledge knowledge) {
		PBTerm knowledgeTerm = knowledge.getPrologTerm();
		if (AT.equals(knowledgeTerm.getName())) {
			String objectID = knowledgeTerm.getArgument(1).getName();
			if (WASTE.equals(objectID))
				return true;
		}
		return false;
	}
	
	public static List<Knowledge> getWastePositions(List<Knowledge> listKB) {
		LinkedList<Knowledge> wastePositions = new LinkedList<Knowledge>();
		for (Knowledge knowledge : listKB) {
			if (isWastePosition(knowledge)) {
				wastePositions.add(knowledge);				
			}
		}
		return wastePositions;
	}

	public static int getAmountOfWaste(Knowledge knowledge) {
		PBTerm prologTerm = knowledge.getPrologTerm();
		PBTerm wasteTerm = prologTerm.getArgument(1);
		return (int)wasteTerm.getArgument(1).intValue();
	}
}
